@@ -46,12 +46,15 @@ class ProductFilter(filters.FilterSet):
     """Product filters"""
 
     store = filters.CharFilter(method="by_store")
+    category = filters.CharFilter(method="by_category")
 
     class Meta:
         model = Product
         fields = [
             "is_featured",
-            "category__name",
+            "type",
+            "store",
+            "category",
         ]
 
     def by_store(self, queryset, name, value):
@@ -64,6 +67,9 @@ class ProductFilter(filters.FilterSet):
             )
         ).filter(product_variants__inventories__store=value)
 
+    def by_category(self, queryset, name, value):
+        return queryset.filter(category__name__iexact=value)
+
 
 class VariantFilter(filters.FilterSet):
     """Variant filters"""
@@ -80,6 +86,7 @@ class CategoryFilter(filters.FilterSet):
     """Category filters"""
 
     store = filters.CharFilter(method="by_store")
+    type = filters.ChoiceFilter(choices=Product.ProductTypes.choices, method="by_type")
 
     class Meta:
         model = Category
@@ -88,14 +95,22 @@ class CategoryFilter(filters.FilterSet):
         ]
 
     def by_store(self, queryset, name, value):
-        return queryset.prefetch_related(
-            Prefetch(
-                "products",
-                queryset=Product.objects.filter(
-                    product_variants__inventories__store=value
-                ),
+        return (
+            queryset.prefetch_related(
+                Prefetch(
+                    "subcategories__products",
+                    queryset=Product.objects.filter(
+                        product_variants__inventories__store=value,
+                        is_active=True,
+                    ),
+                )
             )
-        ).filter(products__product_variants__inventories__store=value)
+            .filter(subcategories__products__product_variants__inventories__store=value)
+            .distinct()
+        )
+
+    def by_type(self, queryset, name, value):
+        return queryset.filter(subcategories__products__type=value)
 
 
 class OrderFilter(filters.FilterSet):
