@@ -4,10 +4,12 @@ from datetime import date, time
 
 from django.conf import settings
 from django.contrib.gis.geos import Point
+from django.core.exceptions import ObjectDoesNotExist
 from django.utils.timezone import now
 from django.utils.translation import gettext_lazy as _
 from rest_framework import serializers
 
+from config import settings as tby_settings
 from ob_dj_store.core.stores.models import (
     Attribute,
     AttributeChoice,
@@ -22,6 +24,8 @@ from ob_dj_store.core.stores.models import (
     Order,
     OrderHistory,
     OrderItem,
+    Payment,
+    PaymentMethod,
     PhoneContact,
     Product,
     ProductAttribute,
@@ -30,6 +34,7 @@ from ob_dj_store.core.stores.models import (
     ProductVariant,
     ShippingMethod,
     Store,
+    Tax,
 )
 from ob_dj_store.core.stores.models._inventory import Inventory
 from ob_dj_store.core.stores.utils import distance
@@ -60,80 +65,22 @@ class AttributeSerializer(serializers.ModelSerializer):
 class InventoryValidationMixin:
     def validate(self, attrs: typing.Dict) -> typing.Dict:
         validated_data = super().validate(attrs)
-        if (
-            validated_data["product_variant"].has_inventory
-            and validated_data["quantity"] < 1
-        ):
-            raise serializers.ValidationError(_("Quantity must be greater than 0."))
-        # validate quantity in inventory
-        stock_quantity = (
-            validated_data["product_variant"]
-            .inventories.get(store=validated_data["store"])
-            .quantity
-        )
-        if (
-            validated_data["product_variant"].has_inventory
-            and validated_data["quantity"] > stock_quantity
-        ):
-            raise serializers.ValidationError(
-                _("Quantity is greater than the stock quantity.")
+        if validated_data["product_variant"].has_inventory:
+            if validated_data["quantity"] < 1:
+                raise serializers.ValidationError(_("Quantity must be greater than 0."))
+            # validate quantity in inventory
+            inventory = validated_data["product_variant"].inventories.get(
+                store=validated_data["store"]
             )
+            if not inventory.is_uncountable:
+                stock_quantity = inventory.quantity
+                if validated_data["quantity"] > stock_quantity:
+                    raise serializers.ValidationError(
+                        _("Quantity is greater than the stock quantity.")
+                    )
         return validated_data
 
 
-class CartItemSerializer(InventoryValidationMixin, serializers.ModelSerializer):
-    class Meta:
-        model = CartItem
-        fields = (
-            "id",
-            "cart",
-            "product_variant",
-            "quantity",
-            "store",
-            "unit_price",
-            "total_price",
-            "notes",
-            "attribute_choices",
-        )
-
-    def create(self, validated_data):
-        return super().create(**validated_data)
-
-
-class CartSerializer(serializers.ModelSerializer):
-    items = CartItemSerializer(many=True)
-
-    class Meta:
-        model = Cart
-        fields = (
-            "customer",
-            "items",
-            "total_price",
-        )
-        read_only_fields = ("id", "total_price")
-
-    def update(self, instance, validated_data):
-        instance.items.all().delete()
-        # update or create instance items
-        for item in validated_data["items"]:
-            cart_item = CartItem.objects.create(
-                cart=instance,
-                product_variant=item["product_variant"],
-                quantity=item["quantity"],
-                notes=item["notes"],
-                store=item["store"],
-            )
-            cart_item.attribute_choices.set(item["attribute_choices"])
-            cart_item.save()
-        return instance
-
-
-class CategorySerializer(serializers.ModelSerializer):
-    class Meta:
-        model = Category
-        fields = ("name", "description", "is_active")
-
-
 class OpeningHourSerializer(serializers.ModelSerializer):
     class Meta:
         model = OpeningHours
@@ -170,7 +117,9 @@ class OrderItemSerializer(InventoryValidationMixin, serializers.ModelSerializer)
             "total_amount",
             "preparation_time",
             "notes",
+            "extra_infos",
             "attribute_choices",
+            "attribute_choices_total_amount",
         )
 
     def create(self, validated_data):
@@ -189,7 +138,6 @@ class OrderItemSerializer(InventoryValidationMixin, serializers.ModelSerializer)
 class OrderSerializer(serializers.ModelSerializer):
     items = OrderItemSerializer(many=True, read_only=True)
     history = OrderHistorySerializer(many=True, read_only=True)
-    cart = serializers.IntegerField(write_only=True)
 
     class Meta:
         model = Order
@@ -200,25 +148,60 @@ class OrderSerializer(serializers.ModelSerializer):
             "payment_method",
             "shipping_address",
             "customer",
+            "type_of_order",
             "status",
             "items",
             "total_amount",
             "preparation_time",
             "history",
-            "cart",
             "pickup_time",
+            "extra_infos",
             "created_at",
             "updated_at",
         )
         extra_kwargs = {
             "customer": {"read_only": True},
             "store": {"read_only": True, "required": False},
+            "type_of_order": {"read_only": True, "required": False},
         }
 
+    def to_representation(self, instance):
+        data = super().to_representation(instance)
+        data["store_name"] = instance.store.name
+        return data
+
+    def _get_store(self):
+        store_pk = self.context["view"].kwargs["store_pk"]
+        try:
+            store = Store.objects.get(pk=store_pk)
+        except ObjectDoesNotExist:
+            raise serializers.ValidationError(_("Store does not exist!"))
+        return store
+
     def validate(self, attrs):
-        # The Cart items must not be empty
         user = self.context["request"].user
-        if not user.cart.items.exists():
+        attrs["store"] = self._get_store()
+        if "extra_infos" in attrs:
+            errors = []
+            data = attrs["extra_infos"]
+            for key in settings.DIGITAL_PRODUCTS_REQUIRED_KEYS:
+                if not (key in data.keys() and len(str(data.get(key)))):
+                    errors.append({key: "This field should be filled."})
+            if len(errors) > 0:
+                raise serializers.ValidationError(errors)
+            try:
+                variant = ProductVariant.objects.get(
+                    id=attrs["extra_infos"]["digital_product"]
+                )
+            except ObjectDoesNotExist:
+                raise serializers.ValidationError(_("Digital product does not exist!"))
+            if variant.product.type == Product.ProductTypes.PHYSICAL:
+                raise serializers.ValidationError(
+                    _("You must not fill extra infos for physical products.")
+                )
+
+        # The Cart items must not be empty
+        elif not "extra_infos" in attrs and not user.cart.items.exists():
             raise serializers.ValidationError(_("The Cart must not be empty"))
 
         if "pickup_time" in attrs:
@@ -258,37 +241,77 @@ class OrderSerializer(serializers.ModelSerializer):
                     _("Pickup time must be between store's opening hours")
                 )
 
+        if "payment_method" in attrs:
+            payment_method = attrs["payment_method"]
+            if payment_method.payment_provider == tby_settings.WALLET:
+                # TODO: after we make each wallet to currency we should change this
+                if "extra_infos" in attrs:
+                    amount = attrs["extra_infos"]["price"]
+                else:
+                    amount = user.cart.total_price
+                if user.wallet.balance < amount:
+                    raise serializers.ValidationError(
+                        {"wallet": _("Insufficient Funds")},
+                    )
         return super().validate(attrs)
 
     def create(self, validated_data: typing.Dict):
-        cart_id = validated_data.pop("cart")
-        cart = Cart.objects.get(pk=cart_id)
-        stores = Store.objects.filter(store_items__cart=cart)
+        user = self.context["request"].user
         orders = []
-        for store in stores:
-            order = Order.objects.create(
-                store=store, customer=cart.customer, **validated_data
+        if "extra_infos" in validated_data:
+            amount = validated_data["extra_infos"]["price"]
+            variant = ProductVariant.objects.get(
+                id=validated_data["extra_infos"]["digital_product"]
+            )
+            order = Order.objects.create(customer=user, **validated_data)
+            order_item = OrderItem.objects.create(
+                order=order,
+                product_variant=variant,
+                quantity=1,
             )
-            items = store.store_items.filter(cart=cart)
-            for item in items:
-                order_item = OrderItem.objects.create(
-                    order=order,
-                    product_variant=item.product_variant,
-                    quantity=item.quantity,
-                )
-                order_item.attribute_choices.set(item.attribute_choices.all())
-            items.delete()
             orders.append(order)
-        return orders
+        else:
+            cart = user.cart
+            amount = cart.total_price
+            stores = Store.objects.filter(store_items__cart=cart)
+            orders = []
+            validated_data.pop("store")
+            for store in stores:
+                order = Order.objects.create(
+                    store=store, customer=cart.customer, **validated_data
+                )
+                items = store.store_items.filter(cart=cart)
+                for item in items:
+                    order_item = OrderItem.objects.create(
+                        order=order,
+                        product_variant=item.product_variant,
+                        quantity=item.quantity,
+                    )
+                    order_item.attribute_choices.set(item.attribute_choices.all())
+                orders.append(order)
+        payment = Payment.objects.create(
+            user=user,
+            amount=amount,
+            method=validated_data.get("payment_method"),
+            currency=settings.DEFAULT_CURRENCY,
+            orders=orders,
+        )
+        return {
+            "orders": orders,
+            "payment_url": payment.payment_url,
+        }
 
-    def update(self, instance: Order, validated_data: typing.Dict):
-        payment_method = validated_data["payment_method"]
-        store = instance.store
-        if payment_method not in store.payment_methods.all():
-            raise serializers.ValidationError(
-                _(f"this payment method is not supported")
-            )
-        return instance
+
+class CreateOrderResponseSerializer(serializers.Serializer):
+    orders = OrderSerializer(many=True, read_only=True)
+    payment_url = serializers.CharField(read_only=True)
+    extra_infos = serializers.JSONField(required=False)
+
+    # write only fields
+    shipping_method = serializers.IntegerField(write_only=True, required=False)
+    payment_method = serializers.IntegerField(write_only=True, required=False)
+    shipping_address = serializers.IntegerField(write_only=True, required=False)
+    pickup_time = serializers.DateTimeField(write_only=True, required=False)
 
 
 class ProductTagSerializer(serializers.ModelSerializer):
@@ -310,35 +333,138 @@ class ProductAttributeSerializer(serializers.ModelSerializer):
         fields = (
             "id",
             "name",
+            "is_mandatory",
             "attribute_choices",
+            "type",
         )
 
 
 class ProductVariantSerializer(serializers.ModelSerializer):
     product_attributes = ProductAttributeSerializer(many=True)
     is_primary = serializers.SerializerMethodField()
+    active_inventories = serializers.SerializerMethodField()
 
     class Meta:
         model = ProductVariant
-        fields = ("id", "name", "sku", "product_attributes", "is_primary")
+        fields = (
+            "id",
+            "name",
+            "sku",
+            "product_attributes",
+            "is_primary",
+            "active_inventories",
+        )
 
     def get_is_primary(self, obj):
         return True if obj.inventories.filter(is_primary=True).exists() else False
 
+    def get_active_inventories(self, obj):
+        if obj.inventories.filter(is_active=True).exists():
+            return obj.inventories.filter(is_active=True).values(
+                "price", "store", "discount_percent"
+            )
+
+
+class CartItemSerializer(InventoryValidationMixin, serializers.ModelSerializer):
+    image = serializers.SerializerMethodField()
+
+    class Meta:
+        model = CartItem
+        fields = (
+            "id",
+            "product_variant",
+            "quantity",
+            "store",
+            "unit_price",
+            "total_price",
+            "notes",
+            "attribute_choices",
+            "extra_infos",
+            "attribute_choices_total_price",
+            "image",
+        )
+        extra_kwargs = {
+            "store": {
+                "required": True,
+            },
+        }
+
+    def validate(self, attrs: typing.Dict) -> typing.Dict:
+        return super(CartItemSerializer, self).validate(attrs)
+
+    def get_image(self, obj):
+        qs = ProductMedia.objects.filter(product=obj.product_variant.product)
+        if qs:
+            return qs.first().image.url
+        else:
+            return None
+
+    def to_representation(self, instance: CartItem):
+        data = super().to_representation(instance)
+        data["product_variant"] = ProductVariantSerializer(
+            instance=instance.product_variant
+        ).data
+        data["product_name"] = instance.product_variant.product.name
+        data["attribute_choices"] = AttributeChoiceSerializer(
+            instance.attribute_choices.all(),
+            many=True,
+        ).data
+        return data
+
+    def create(self, validated_data):
+        return super().create(**validated_data)
+
+
+class CartSerializer(serializers.ModelSerializer):
+    items = CartItemSerializer(many=True)
+
+    class Meta:
+        model = Cart
+        fields = (
+            "customer",
+            "items",
+            "total_price",
+            "tax_amount",
+            "total_price_with_tax",
+        )
+        read_only_fields = (
+            "id",
+            "total_price",
+            "tax_amount",
+            "total_price_with_tax",
+        )
+
+    def update(self, instance, validated_data):
+        instance.items.all().delete()
+        # update or create instance items
+        for item in validated_data["items"]:
+            attribute_choices = item.pop("attribute_choices", None)
+            cart_item = CartItem.objects.create(
+                cart=instance,
+                **item,
+            )
+            if attribute_choices:
+                cart_item.attribute_choices.set(attribute_choices)
+            cart_item.save()
+        return instance
+
 
 class ProductMediaSerializer(serializers.ModelSerializer):
+    image_thumbnail = serializers.ImageField(read_only=True)
+
     class Meta:
         model = ProductMedia
         fields = (
             "id",
             "is_primary",
             "image",
+            "image_thumbnail",
             "order_value",
         )
 
 
 class ProductSerializer(serializers.ModelSerializer):
-    variants = ProductVariantSerializer(many=True, source="product_variants")
+    product_variants = ProductVariantSerializer(many=True)
     product_images = ProductMediaSerializer(many=True, source="images")
     is_favorite = serializers.SerializerMethodField()
     default_variant = ProductVariantSerializer(read_only=True, many=False)
@@ -352,7 +478,7 @@ class ProductSerializer(serializers.ModelSerializer):
             "description",
             "is_favorite",
             "product_images",
-            "variants",
+            "product_variants",
             "default_variant",
         )
 
@@ -385,16 +511,26 @@ class ProductListSerializer(ProductSerializer):
             "description",
             "product_images",
             "product_variants",
+            "type",
             "default_variant",
         )
 
 
 class SubCategorySerializer(serializers.ModelSerializer):
     products = ProductListSerializer(many=True)
+    image_thumbnail = serializers.ImageField(read_only=True)
 
     class Meta:
         model = Category
-        fields = ("id", "name", "description", "is_active", "products")
+        fields = (
+            "id",
+            "name",
+            "description",
+            "is_active",
+            "products",
+            "image",
+            "image_thumbnail",
+        )
 
 
 class CategorySerializer(serializers.ModelSerializer):
@@ -507,11 +643,11 @@ class StoreSerializer(serializers.ModelSerializer):
     opening_hours = OpeningHourSerializer(many=True, read_only=True)
     phone_contacts = serializers.SerializerMethodField()
     in_range_delivery = serializers.SerializerMethodField()
+    is_closed = serializers.SerializerMethodField()
     is_favorite = serializers.SerializerMethodField()
     address_line = serializers.SerializerMethodField()
     shipping_methods = ShippingMethodSerializer(many=True, read_only=True)
     distance = serializers.SerializerMethodField()
-    inventories = InventorySerializer(many=True, read_only=True, required=False)
     current_day_opening_hours = serializers.SerializerMethodField()
 
     class Meta:
@@ -531,14 +667,27 @@ class StoreSerializer(serializers.ModelSerializer):
             "min_free_delivery_amount",
             "opening_hours",
             "in_range_delivery",
+            "is_closed",
             "is_favorite",
             "created_at",
             "updated_at",
-            "inventories",
             "phone_contacts",
             "current_day_opening_hours",
         )
 
+    def get_is_closed(self, obj):
+        current_time = now()
+        current_op_hour = obj.opening_hours.filter(
+            weekday=current_time.weekday()
+        ).first()
+        if current_op_hour:
+            return (
+                not current_op_hour.from_hour
+                <= current_time.time()
+                <= current_op_hour.to_hour
+            )
+        return True
+
     def get_in_range_delivery(self, obj):
         user_location = self.context["request"].query_params.get("point")
         if user_location and obj.poly:
@@ -587,3 +736,44 @@ class StoreSerializer(serializers.ModelSerializer):
                 op_hour["from_hour"] = settings.DEFAULT_OPENING_HOURS[0]["from_hour"]
                 op_hour["to_hour"] = settings.DEFAULT_OPENING_HOURS[0]["to_hour"]
                 return op_hour
+
+    def to_representation(self, instance):
+        data = super().to_representation(instance)
+        view = self.context.get("view")
+        if view:
+            if view.action == "list":
+                return data
+        data["inventories"] = InventorySerializer(
+            instance.inventories.all(), many=True
+        ).data
+        return data
+
+
+class PaymentMethodSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = PaymentMethod
+        fields = (
+            "id",
+            "payment_provider",
+            "name",
+            "description",
+        )
+
+
+class PaymentSerializer(serializers.ModelSerializer):
+    orders = OrderSerializer(many=True, read_only=True)
+
+    class Meta:
+        model = Payment
+        fields = ("id", "method", "orders", "amount", "currency", "payment_post_at")
+
+
+class TaxSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = Tax
+        fields = (
+            "id",
+            "rate",
+            "name",
+            "value",
+        )
