@@ -1,16 +1,20 @@
 import logging
 import typing
 
+from django.core.exceptions import ObjectDoesNotExist
 from django.db.models import Prefetch
 from django.utils.decorators import method_decorator
 from django.utils.translation import ugettext_lazy as _
+from django_auto_prefetching import AutoPrefetchViewSetMixin, prefetch
 from django_filters.rest_framework import DjangoFilterBackend
 from drf_yasg.utils import swagger_auto_schema
 from rest_framework import mixins, permissions, status, viewsets
 from rest_framework.decorators import action
+from rest_framework.exceptions import ValidationError
 from rest_framework.request import Request
 from rest_framework.response import Response
 from rest_framework_gis.filters import DistanceToPointOrderingFilter
+from rest_framework_nested.viewsets import NestedViewSetMixin
 
 from ob_dj_store.apis.stores.filters import (
     CategoryFilter,
@@ -21,27 +25,36 @@ from ob_dj_store.apis.stores.filters import (
     VariantFilter,
 )
 from ob_dj_store.apis.stores.rest.serializers.serializers import (
+    CartItemSerializer,
     CartSerializer,
     CategorySerializer,
+    CreateOrderResponseSerializer,
     FeedbackSerializer,
     InventorySerializer,
     OrderSerializer,
+    PaymentMethodSerializer,
+    PaymentSerializer,
     ProductListSerializer,
     ProductSerializer,
     ProductVariantSerializer,
+    ShippingMethodSerializer,
     StoreSerializer,
+    TaxSerializer,
 )
-from ob_dj_store.apis.tap.serializers import TapPaymentSerializer
-from ob_dj_store.core.stores.gateway.tap.models import TapPayment
 from ob_dj_store.core.stores.models import (
     Cart,
+    CartItem,
     Category,
     Favorite,
     FeedbackConfig,
     Order,
+    Payment,
+    PaymentMethod,
     Product,
     ProductVariant,
+    ShippingMethod,
     Store,
+    Tax,
 )
 from ob_dj_store.core.stores.models._inventory import Inventory
 
@@ -49,7 +62,10 @@ logger = logging.getLogger(__name__)
 
 
 class StoreView(
-    mixins.ListModelMixin, mixins.RetrieveModelMixin, viewsets.GenericViewSet
+    AutoPrefetchViewSetMixin,
+    mixins.ListModelMixin,
+    mixins.RetrieveModelMixin,
+    viewsets.GenericViewSet,
 ):
     serializer_class = StoreSerializer
     permission_classes = [
@@ -93,8 +109,13 @@ class StoreView(
                     ],
                 )
                 .order_by("-orders__created_at")
+                .distinct()
             )
-        return queryset
+        queryset.select_related(
+            "phone_contacts",
+            "opening_hours",
+        )
+        return prefetch(queryset, self.serializer_class)
 
     @swagger_auto_schema(
         operation_summary="List Stores",
@@ -223,6 +244,55 @@ class CartView(
         return super().update(request, *args, **kwargs)
 
 
+class CartItemView(
+    mixins.RetrieveModelMixin,
+    mixins.UpdateModelMixin,
+    mixins.DestroyModelMixin,
+    viewsets.GenericViewSet,
+):
+    serializer_class = CartItemSerializer
+    permission_classes = [
+        permissions.IsAuthenticated,
+    ]
+    queryset = CartItem.objects.all()
+
+    @swagger_auto_schema(
+        operation_summary="Retrieve Cart Item",
+        operation_description="""
+            Retrieve the current cart's cart item
+        """,
+        tags=[
+            "CartItem",
+        ],
+    )
+    def retrieve(self, request, *args, **kwargs):
+        return super().retrieve(request, *args, **kwargs)
+
+    @swagger_auto_schema(
+        operation_summary="Update Customer Cart Item",
+        operation_description="""
+            Updates the current customer's cart item
+        """,
+        tags=[
+            "CartItem",
+        ],
+    )
+    def update(self, request, *args, **kwargs):
+        return super().update(request, *args, **kwargs)
+
+    @swagger_auto_schema(
+        operation_summary="Delete Customer Cart Item",
+        operation_description="""
+            Deletes the current customer's cart item
+        """,
+        tags=[
+            "CartItem",
+        ],
+    )
+    def delete(self, request, *args, **kwargs):
+        return self.destroy(request, *args, **kwargs)
+
+
 class OrderView(
     mixins.ListModelMixin,
     mixins.RetrieveModelMixin,
@@ -236,6 +306,11 @@ class OrderView(
     filterset_class = OrderFilter
     queryset = Order.objects.all()
 
+    def get_serializer_class(self):
+        if self.action == "create":
+            return CreateOrderResponseSerializer
+        return self.serializer_class
+
     def get_queryset(self):
         queryset = super().get_queryset()
         return queryset.filter(customer=self.request.user)
@@ -277,12 +352,19 @@ class OrderView(
         ],
     )
     def create(self, request, *args, **kwargs):
-        serializer = self.get_serializer(data=request.data)
+        context = self.get_serializer_context()
+        serializer = OrderSerializer(data=request.data, context=context)
         orders = []
         if serializer.is_valid(raise_exception=True):
-            orders = serializer.save()
-        data = OrderSerializer(orders, many=True).data
-        return Response(status=status.HTTP_201_CREATED, data=data)
+            orders, payment_url = serializer.save().values()
+        orders_data = OrderSerializer(orders, many=True).data
+        return Response(
+            status=status.HTTP_201_CREATED,
+            data={
+                "payment_url": payment_url,
+                "orders": orders_data,
+            },
+        )
 
     @action(
         methods=["POST"],
@@ -336,9 +418,15 @@ class VariantView(mixins.ListModelMixin, viewsets.GenericViewSet):
 
 
 class ProductView(
-    mixins.ListModelMixin, mixins.RetrieveModelMixin, viewsets.GenericViewSet
+    NestedViewSetMixin,
+    mixins.ListModelMixin,
+    mixins.RetrieveModelMixin,
+    viewsets.GenericViewSet,
 ):
     serializer_class = ProductSerializer
+    parent_lookup_kwargs = {
+        "store_pk": "product_variants__inventories__store__pk",
+    }
     permission_classes = [
         permissions.AllowAny,
     ]
@@ -528,16 +616,16 @@ class TransactionsViewSet(
     mixins.ListModelMixin,
     viewsets.GenericViewSet,
 ):
-    serializer_class = TapPaymentSerializer
+    serializer_class = PaymentSerializer
     permission_classes = [
         permissions.IsAuthenticated,
     ]
-    queryset = TapPayment.objects.all()
+    queryset = Payment.objects.all()
 
     def get_queryset(self):
-        return TapPayment.objects.filter(
-            user=self.request.user, status=TapPayment.Status.CAPTURED.value
-        )
+        return Payment.objects.filter(
+            user=self.request.user, status=Payment.PaymentStatus.SUCCESS.value
+        ).order_by("payment_post_at")
 
     @swagger_auto_schema(
         operation_summary="List Users's Captured Transactions",
@@ -550,3 +638,91 @@ class TransactionsViewSet(
     )
     def list(self, request, *args, **kwargs):
         return super().list(request, *args, **kwargs)
+
+
+class PaymentMethodViewSet(
+    mixins.ListModelMixin,
+    viewsets.GenericViewSet,
+):
+    serializer_class = PaymentMethodSerializer
+    permission_classes = [
+        permissions.IsAuthenticated,
+    ]
+    queryset = PaymentMethod.objects.all()
+
+    def get_queryset(self):
+        try:
+            store = Store.objects.get(pk=self.kwargs["store_pk"])
+        except ObjectDoesNotExist:
+            raise ValidationError(_(f"Store does not Exist"))
+        return store.payment_methods.filter(is_active=True)
+
+    @swagger_auto_schema(
+        operation_summary="List Payment Methods",
+        operation_description="""
+            List Payment methods
+        """,
+        tags=[
+            "Payment Method",
+        ],
+    )
+    def list(self, request, *args, **kwargs):
+        return super().list(request, *args, **kwargs)
+
+
+class ShippingMethodViewSet(
+    mixins.ListModelMixin,
+    viewsets.GenericViewSet,
+):
+    serializer_class = ShippingMethodSerializer
+    permission_classes = [
+        permissions.IsAuthenticated,
+    ]
+    queryset = ShippingMethod.objects.all()
+
+    def get_queryset(self):
+        """
+        if we get store_pk=0 we fetch all the shipping methods
+        """
+        if self.kwargs["store_pk"] == 0:
+            return ShippingMethod.objects.filter(is_active=True)
+        try:
+            store = Store.objects.get(pk=self.kwargs["store_pk"])
+        except ObjectDoesNotExist:
+            raise ValidationError(_(f"Store does not Exist"))
+        return store.shipping_methods.filter(is_active=True)
+
+    @swagger_auto_schema(
+        operation_summary="List Shipping Methods",
+        operation_description="""
+            List Shipping methods
+        """,
+        tags=[
+            "Store",
+        ],
+    )
+    def list(self, request, *args, **kwargs):
+        return super().list(request, *args, **kwargs)
+
+
+class TaxViewSet(
+    mixins.ListModelMixin,
+    viewsets.GenericViewSet,
+):
+    serializer_class = TaxSerializer
+    permission_classes = [
+        permissions.IsAuthenticated,
+    ]
+    queryset = Tax.objects.all()
+
+    @swagger_auto_schema(
+        operation_summary="List Taxes",
+        operation_description="""
+            List Taxes
+        """,
+        tags=[
+            "Tax",
+        ],
+    )
+    def list(self, request, *args, **kwargs):
+        return super().list(request, *args, **kwargs)
