@@ -28,6 +28,7 @@ class TapPayment(models.Model):
     class Sources(models.TextChoices):
         CREDIT_CARD = "src_card", _("Credit Card")
         KNET = "src_kw.knet", _("KNet")
+        ALL = "src_all", _("All")
 
     status = models.CharField(max_length=100, choices=Status.choices)
     source = models.CharField(max_length=100, choices=Sources.choices)
@@ -48,6 +49,7 @@ class TapPayment(models.Model):
     payment = models.OneToOneField(
         "stores.Payment",
         on_delete=models.CASCADE,
+        related_name="tap_payment",
         help_text=_("Reference local payment transaction table"),
     )
     result = models.CharField(
@@ -99,4 +101,11 @@ class TapPayment(models.Model):
             self.callback_response = tap_payload
             self.status = tap_payload["status"]
             self.save()
+            self.mark_transaction()
             logger.info(f"Mark TAP Transaction finished for charge ID {self.charge_id}")
+
+    def mark_transaction(self):
+        if self.status == self.Status.CAPTURED:
+            self.payment.mark_paid()
+        elif self.status == self.Status.FAILED:
+            self.payment.mark_failed()
