@@ -1,5 +1,11 @@
+import typing
+
 from django.contrib.contenttypes.models import ContentType
+from django.core.exceptions import ObjectDoesNotExist, ValidationError
 from django.db import models
+from django.utils.translation import gettext_lazy as _
+
+from config import settings
 
 
 class ActiveMixin:
@@ -41,7 +47,36 @@ class PaymentMethodManager(ActiveMixin, models.Manager):
 
 
 class PaymentManager(models.Manager):
-    pass
+    def create(self, *args: typing.Any, **kwargs: typing.Any):
+        from ob_dj_store.core.stores.gateway.tap.models import TapPayment
+        from ob_dj_store.core.stores.models import Tax, WalletTransaction
+
+        orders = kwargs.pop("orders", [])
+        try:
+            kwargs["payment_tax"] = Tax.objects.get(is_active=True)
+        except ObjectDoesNotExist:
+            raise ValidationError(_("Tax object is missing"), code="invalid")
+        gateway = settings.DEFAULT_PAYMENT_METHOD
+        method = kwargs.get("method", None)
+        if method:
+            gateway = method.payment_provider
+        instance: "models.Payment" = super().create(*args, **kwargs)
+        instance.orders.set(orders)
+        if gateway in [settings.TAP_CREDIT_CARD, settings.TAP_KNET, settings.TAP_ALL]:
+            source = gateway
+            TapPayment.objects.create(
+                source=source,
+                payment=instance,
+                user=kwargs.get("user"),
+            )
+        elif gateway == settings.WALLET:
+            WalletTransaction.objects.create(
+                wallet=kwargs["user"].wallet,
+                type=WalletTransaction.TYPE.DEBIT,
+                amount=kwargs["amount"],
+            )
+            instance.mark_paid()
+        return instance
 
 
 class InventoryManager(ActiveMixin, models.Manager):
@@ -133,3 +168,14 @@ class FeedbackAttributeManager(models.Manager):
 class ProductVariantStoreManager(models.Manager):
     def create(self, *args, **kwargs):
         return super().create(*args, **kwargs)
+
+
+class WalletTransactionManager(models.Manager):
+    def create(self, *args, **kwargs):
+        from ob_dj_store.core.stores.models._wallet import WalletTransaction
+
+        wallet = kwargs["wallet"]
+        type = kwargs["type"]
+        if type == WalletTransaction.TYPE.DEBIT and wallet.balance < kwargs["amount"]:
+            raise ValidationError(_("Insufficient Funds"))
+        return super().create(*args, **kwargs)
