@@ -6,8 +6,9 @@ from ob_dj_store.core.stores.models._feedback import (
     FeedbackAttribute,
     FeedbackConfig,
 )
+from ob_dj_store.core.stores.models._inventory import Inventory, InventoryOperations
 from ob_dj_store.core.stores.models._order import Order, OrderHistory, OrderItem
-from ob_dj_store.core.stores.models._payment import Payment
+from ob_dj_store.core.stores.models._payment import Payment, Tax
 from ob_dj_store.core.stores.models._product import (
     Attribute,
     AttributeChoice,
@@ -25,6 +26,7 @@ from ob_dj_store.core.stores.models._store import (
     ShippingMethod,
     Store,
 )
+from ob_dj_store.core.stores.models._wallet import Wallet, WalletTransaction
 
 __all__ = [
     "Store",
@@ -52,4 +54,9 @@ __all__ = [
     "FeedbackAttribute",
     "FeedbackConfig",
     "PhoneContact",
+    "Inventory",
+    "InventoryOperations",
+    "Tax",
+    "WalletTransaction",
+    "Wallet",
 ]
