@@ -2,19 +2,20 @@ from django.conf import settings
 from django.db.models.signals import post_save
 from django.dispatch import receiver
 
-from ob_dj_store.core.stores.models import Cart, Order, OrderHistory
+from ob_dj_store.core.stores.models import Cart, Order, OrderHistory, Wallet
 
 
 @receiver(
     post_save,
     sender=settings.AUTH_USER_MODEL,
-    dispatch_uid="create_customer_cart_handler",
+    dispatch_uid="create_customer_cart_and_wallet_handler",
 )
-def create_customer_cart_handler(sender, instance, created, **kwargs):
+def create_customer_cart_and_wallet_handler(sender, instance, created, **kwargs):
     if not created:
         return
     cart = Cart(customer=instance)
     cart.save()
+    Wallet.objects.create(user=instance)
 
 
 # add receiver to ProductVariant to create inventory
