@@ -20,3 +20,9 @@ def import_class_from_string(serializer_class_name):
 def product_media_upload_to(instance, filename):
     ext = filename.split(".")[-1]
     return f"product_media/{instance.product.id}_{instance.order_value}_{int(now().timestamp())}.{ext}"
+
+
+def category_media_upload_to(instance, filename):
+    ext = filename.split(".")[-1]
+    if instance:
+        return f"category_media/{instance.name}_{instance.order_value}_{int(now().timestamp())}.{ext}"
